/*************************************************************************\
*                                                                         *
*    File    : data_mux.v                                                 *
*    Purpose : Data Multiplexer Module                                    *
*    Author  : Novan Hartadi (novan@vlsi.itb.ac.id)                       *
*                                                                         *
/*************************************************************************\

/*************************************************************************\
*                                                                         *
*    Brief Description :                                                  *
*                                                                         *
*    The function of this module is to determine which data nibbles will  *
*    be transmitted to MII, which are related to data_select signal from  *
*    state machine. Data nibble 4'b0101 is pattern for Preamble, JAM, and *
*    PAD. While data nibble 4'b1101 is pattern for SFD.                   *
*                                                                         *
\*************************************************************************/

`define PATTERN_PREAMBLE (4'b0101)
`define PATTERN_SFD      (4'b1101)

module data_mux (
  data_select,
  data_in,
  crc,
  data_out
  );

input [1:0] data_select;
input [3:0] data_in;
input [3:0] crc;
output [3:0] data_out;

reg [3:0] data_out;

always @(data_select or data_in or crc)
  case (data_select)
    2'b00   : data_out = data_in;
    2'b01   : data_out = `PATTERN_PREAMBLE;
    2'b10   : data_out = `PATTERN_SFD;
    2'b11   : data_out = crc;
    default : data_out = 4'h0;
  endcase

endmodule
