/*************************************************************************\
*                                                                         *
*    File    : fifo_synch.v                                               *
*    Purpose : FIFO Synchronization Module                                *
*    Author  : Novan Hartadi (novan@vlsi.itb.ac.id)                       *
*                                                                         *
/*************************************************************************\

/*************************************************************************\
*                                                                         *
*    Brief Description :                                                  *
*                                                                         *
*    The function of this module is to keep synchronization all outputs   *
*    with the rising edge of input clock. FIFO will transmit a valid data *
*    nibble output if transmit_enable is asserted.                        *
*    When transmit_enable is asserted, tx_en will be asserted. If         *
*    transmit_error is asserted tx_er will be asserted too.               *
*                                                                         *
\*************************************************************************/

module fifo_synch (
  data,
  transmit_enable,
  transmit_error,
  clk,
  txd,
  tx_en,
  tx_er
  );

input [3:0] data;
input transmit_enable;
input transmit_error;
input clk;
output [3:0] txd;
output tx_en;
output tx_er;

reg [3:0] txd;
reg tx_en;
reg tx_er;

always @(posedge clk)
  if (transmit_enable)
    begin
      txd <= data;
      tx_en <= 1'b1;
      tx_er <= transmit_error;
    end
  else
    begin
      txd <= 4'h0;
      tx_en <= 1'b0;
      tx_er <= 1'b0;
    end

endmodule
