/*************************************************************************\
*                                                                         *
*    File    : random_number_gen.v                                        *
*    Purpose : Random Number Generator Module                             *
*    Author  : Novan Hartadi (novan@vlsi.itb.ac.id)                       *
*                                                                         *
/*************************************************************************\

/*************************************************************************\
*                                                                         *
*    Brief Description :                                                  *
*                                                                         *
*    This module generates random number using Linear Feedback Shift      *
*    Register which contains 10 bit register with polinomial feedback is  *
*    feedback = x[2] XNOR x[9].                                           *
*    This Random Number Generator will select random number from range    *
*    between 0 to (2^k)-1 with k is the smaller number between number of  *
*    collision and 10. The number of collision is equal to coll_attempt.  *
*                                                                         *
\*************************************************************************/

module random_number_gen (
  coll_attempt,
  clk,
  reset_n,
  random
  );

input [3:0] coll_attempt;
input clk;
input reset_n;
output [9:0] random;

wire [9:0] x;
wire feedback;

dff_tx u0 ( .d(feedback), .clk(clk), .reset_n(reset_n), .q(x[0]) );
dff_tx u1 ( .d(x[0]), .clk(clk), .reset_n(reset_n), .q(x[1]) );
dff_tx u2 ( .d(x[1]), .clk(clk), .reset_n(reset_n), .q(x[2]) );
dff_tx u3 ( .d(x[2]), .clk(clk), .reset_n(reset_n), .q(x[3]) );
dff_tx u4 ( .d(x[3]), .clk(clk), .reset_n(reset_n), .q(x[4]) );
dff_tx u5 ( .d(x[4]), .clk(clk), .reset_n(reset_n), .q(x[5]) );
dff_tx u6 ( .d(x[5]), .clk(clk), .reset_n(reset_n), .q(x[6]) );
dff_tx u7 ( .d(x[6]), .clk(clk), .reset_n(reset_n), .q(x[7]) );
dff_tx u8 ( .d(x[7]), .clk(clk), .reset_n(reset_n), .q(x[8]) );
dff_tx u9 ( .d(x[8]), .clk(clk), .reset_n(reset_n), .q(x[9]) );

assign feedback = x[2] ~^ x[9];

assign random [0] = (coll_attempt > 0) ? x [0] : 1'b0;
assign random [1] = (coll_attempt > 1) ? x [1] : 1'b0;
assign random [2] = (coll_attempt > 2) ? x [2] : 1'b0;
assign random [3] = (coll_attempt > 3) ? x [3] : 1'b0;
assign random [4] = (coll_attempt > 4) ? x [4] : 1'b0;
assign random [5] = (coll_attempt > 5) ? x [5] : 1'b0;
assign random [6] = (coll_attempt > 6) ? x [6] : 1'b0;
assign random [7] = (coll_attempt > 7) ? x [7] : 1'b0;
assign random [8] = (coll_attempt > 8) ? x [8] : 1'b0;
assign random [9] = (coll_attempt > 9) ? x [9] : 1'b0;

endmodule